"""
Unit tests for form_factory helper functions and _ENUM_OPTION_TYPES.
"""

from enum import Enum
from typing import Any, cast
from unittest.mock import MagicMock

import pytest
from babel.support import Translations
from fields_metadata import FieldMetadata
from wtforms import SelectField

from orama.enums import (
    GeoProjection,
    LineInterpolation,
    LinePlacement,
    LineSortOrder,
    SortOrder,
)
from orama.options import (
    GeoProjectionOption,
    LineInterpolationOption,
    LinePlacementOption,
    LineSortOrderOption,
    SortOrderOption,
    StringOption,
)
from orama.strategies.bar import BarChartStrategy
from orama.web.form_factory import (
    _ENUM_OPTION_TYPES,
    LBL_PREFIX,
    OPT_PREFIX,
    VAR_PREFIX,
    _add_option_field,
    _add_variable_field,
    create_figure_form_class,
)


def test_enum_option_types_keys() -> None:
    expected = {SortOrderOption, LinePlacementOption, LineInterpolationOption, LineSortOrderOption, GeoProjectionOption}
    assert set(_ENUM_OPTION_TYPES.keys()) == expected


def test_enum_option_types_sort_order_has_coerce() -> None:
    _, extra = _ENUM_OPTION_TYPES[SortOrderOption]
    assert extra == {"coerce": int}


@pytest.mark.parametrize(
    "opt_type,enum_cls",
    [
        (SortOrderOption, SortOrder),
        (LinePlacementOption, LinePlacement),
        (LineInterpolationOption, LineInterpolation),
        (LineSortOrderOption, LineSortOrder),
        (GeoProjectionOption, GeoProjection),
    ],
)
def test_add_option_field_enum_produces_select_field(
    opt_type: type,
    enum_cls: type[Enum],
) -> None:
    option = opt_type(
        name="Test",
        description="Test option",
        map_to="test_field",
        default_value=list(enum_cls)[0],
    )
    opt_desc = option.describe()
    attrs: dict[str, Any] = {}
    _add_option_field(attrs, opt_desc)
    key = f"{OPT_PREFIX}test_field"
    assert key in attrs
    assert attrs[key].field_class is SelectField
    expected_values = [e.value for e in enum_cls]
    field_values = [choice[0] for choice in attrs[key].kwargs["choices"]]
    assert field_values == expected_values


def test_create_figure_form_class_bar_strategy() -> None:
    cat_meta = MagicMock(spec=FieldMetadata)
    cat_meta.categorical = True
    cat_meta.numeric = False
    cat_meta.derived = False
    cat_meta.effective_type = str
    cat_meta.extra = {}
    num_meta = MagicMock(spec=FieldMetadata)
    num_meta.categorical = False
    num_meta.numeric = True
    num_meta.derived = False
    num_meta.effective_type = float
    num_meta.extra = {}
    fields_metadata = cast(dict[str, FieldMetadata], {"category": cat_meta, "value": num_meta})
    form_cls = create_figure_form_class(BarChartStrategy, fields_metadata)
    from wtforms import Form

    assert issubclass(form_cls, Form)
    form_cls()


@pytest.fixture
def wrapping_tr() -> MagicMock:
    mock = MagicMock(spec=Translations)
    mock.gettext.side_effect = lambda s: f"[{s}]"
    return mock


@pytest.fixture
def bar_fields_metadata() -> dict[str, MagicMock]:
    cat_meta = MagicMock(spec=FieldMetadata)
    cat_meta.categorical = True
    cat_meta.numeric = False
    cat_meta.derived = False
    cat_meta.effective_type = str
    cat_meta.extra = {}
    num_meta = MagicMock(spec=FieldMetadata)
    num_meta.categorical = False
    num_meta.numeric = True
    num_meta.derived = False
    num_meta.effective_type = float
    num_meta.extra = {}
    return {"category": cat_meta, "value": num_meta}


def test_create_figure_form_class_translates_fixed_labels(
    wrapping_tr: MagicMock,
    bar_fields_metadata: dict[str, MagicMock],
) -> None:
    fields_metadata = cast(dict[str, FieldMetadata], bar_fields_metadata)
    form_cls = create_figure_form_class(BarChartStrategy, fields_metadata, tr=wrapping_tr)
    form = form_cls()
    assert form[f"{LBL_PREFIX}title"].label.text == "[Title]"
    assert form[f"{LBL_PREFIX}subtitle"].label.text == "[Subtitle]"
    assert form[f"{LBL_PREFIX}caption"].label.text == "[Caption]"
    assert form[f"{LBL_PREFIX}description"].label.text == "[Description]"
    assert form[f"{LBL_PREFIX}storytelling_context"].label.text == "[Storytelling Context]"


def test_create_figure_form_class_no_tr_keeps_original_labels(
    bar_fields_metadata: dict[str, MagicMock],
) -> None:
    fields_metadata = cast(dict[str, FieldMetadata], bar_fields_metadata)
    form_cls = create_figure_form_class(BarChartStrategy, fields_metadata)
    form = form_cls()
    assert form[f"{LBL_PREFIX}title"].label.text == "Title"
    assert form[f"{LBL_PREFIX}subtitle"].label.text == "Subtitle"


def test_add_option_field_translates_enum_labels(wrapping_tr: MagicMock) -> None:
    option = SortOrderOption(
        name="Sort Order",
        description="Sort direction",
        map_to="sort_order",
        default_value=SortOrder.NONE,
    )
    opt_desc = option.describe()
    attrs: dict[str, Any] = {}
    _add_option_field(attrs, opt_desc, tr=wrapping_tr)
    key = f"{OPT_PREFIX}sort_order"
    assert attrs[key].args[0] == "[Sort Order]"
    display_names = [label for _, label in attrs[key].kwargs["choices"]]
    assert all(name.startswith("[") and name.endswith("]") for name in display_names)


def test_add_option_field_translates_string_option_label(wrapping_tr: MagicMock) -> None:
    option = StringOption(
        name="My Label",
        description="A string option",
        map_to="my_label",
    )
    opt_desc = option.describe()
    attrs: dict[str, Any] = {}
    _add_option_field(attrs, opt_desc, tr=wrapping_tr)
    key = f"{OPT_PREFIX}my_label"
    assert attrs[key].args[0] == "[My Label]"


def test_add_variable_field_aggregation_includes_count(
    bar_fields_metadata: dict[str, MagicMock],
) -> None:
    from therismos.grouping import AggregationFunction

    from orama.variables import NumericalAggregationVariable

    fields_metadata = cast(dict[str, FieldMetadata], bar_fields_metadata)
    var_desc = NumericalAggregationVariable(
        name="Value",
        description="A numerical aggregation variable",
        map_to="value_variable",
    ).describe(fields_metadata)
    attrs: dict[str, Any] = {}
    _add_variable_field(attrs, var_desc, fields_metadata)
    function_key = f"{VAR_PREFIX}value_variable__function"
    assert function_key in attrs
    function_values = [value for value, _ in attrs[function_key].kwargs["choices"]]
    assert AggregationFunction.COUNT.value in function_values


def test_add_variable_field_translates_label_and_none_placeholder(
    wrapping_tr: MagicMock,
    bar_fields_metadata: dict[str, MagicMock],
) -> None:
    from orama.variables import CategoricalVariable

    fields_metadata = cast(dict[str, FieldMetadata], bar_fields_metadata)
    var_desc = CategoricalVariable(
        name="Category",
        description="A category variable",
        map_to="category_variable",
        optional=True,
    ).describe(fields_metadata)
    attrs: dict[str, Any] = {}
    _add_variable_field(attrs, var_desc, fields_metadata, tr=wrapping_tr)
    key = f"{VAR_PREFIX}category_variable"
    assert attrs[key].args[0] == "[Category]"
    none_labels = [label for value, label in attrs[key].kwargs["choices"] if value == ""]
    assert none_labels == ["[— none —]"]



"""
Defines themes for figures.
"""

import copy
from collections.abc import Callable

import armonia
import plotly.graph_objects as go
import plotly.io as pio
from polychromos.color import HSLColor
from polychromos.palette import HSLColorSequence


class Theme:
    """
    Encapsulates a theme for figures.
    """

    def __init__(
        self,
        colors: dict[str, HSLColor],
    ) -> None:
        self._armonia_theme = armonia.Theme()
        self.__setup_colors(colors)

    def __setup_colors(
        self,
        colors: dict[str, HSLColor],
    ) -> None:
        for name, color in colors.items():
            self.armonia_theme.set_color(name, color)

        def lightness_variation_based_on_background(
            base_color_name: str,
            background_color_name: str,
            delta: float,
        ) -> Callable[[armonia.Theme], HSLColor]:
            def variation_function(
                theme: armonia.Theme,
            ) -> HSLColor:
                base_color: HSLColor = theme.get_color(base_color_name)
                background_color: HSLColor = theme.get_color(background_color_name)
                amount: float = background_color.lightness - base_color.lightness
                return base_color.delta(delta * 0.02, 0.0, delta * amount)

            return variation_function

        if "background" in colors:
            self.armonia_theme.set_computed_color(
                "background_inverted",
                armonia.colorfunctions.invert("background"),
            )
            self.armonia_theme.set_computed_color(
                "background_inverted_mild",
                armonia.colorfunctions.softer("background_inverted", "background", 0.3),
            )
            self.armonia_theme.set_computed_color(
                "background_inverted_milder",
                armonia.colorfunctions.softer("background_inverted", "background", 0.6),
            )
            self.armonia_theme.set_computed_color(
                "background_inverted_faint",
                armonia.colorfunctions.softer("background_inverted", "background", 0.9),
            )
        for base_color_prefix in ["primary", "secondary", "accent"]:
            if f"{base_color_prefix}_color_normal" in colors:
                self.armonia_theme.set_computed_color(
                    f"{base_color_prefix}_color_softer",
                    lightness_variation_based_on_background(f"{base_color_prefix}_color_normal", "background", 0.4),
                )
                self.armonia_theme.set_computed_color(
                    f"{base_color_prefix}_color_stronger",
                    lightness_variation_based_on_background(f"{base_color_prefix}_color_normal", "background", -0.4),
                )

    @property
    def armonia_theme(self) -> armonia.Theme:
        """
        Gets the underlying armonia theme.

        :return: The armonia theme.
        :rtype: armonia.Theme
        """
        return self._armonia_theme

    def get_color(
        self,
        name_or_value: str,
    ) -> HSLColor:
        """
        Gets a color from the theme by name or hex value.

        :param name_or_value: The name or value of the color.
        :type name_or_value: str
        :raises ValueError: When the color is not in hex value nor found in the theme or web colors.
        :return: The color mapped to the name or value given.
        :rtype: HSLColor
        """
        return self.armonia_theme.get_color(name_or_value)

    def get_all_colors(self) -> list[tuple[str, HSLColor]]:
        """
        Gets all colors defined in the theme.

        :return: A list of tuples with color names and their corresponding colors.
        :rtype: list[tuple[str, HSLColor]]
        """
        return [(c.name, c.color) for c in self.armonia_theme.get_all_colors(sort_by="name")]

    def get_default_color_sequence(self) -> HSLColorSequence:
        """
        Gets the default color sequence for the theme.

        :return: A color sequence to be used as default.
        :rtype: HSLColorSequence
        """
        return [
            self.armonia_theme.get_color("primary_color_normal"),
        ]

    def plotly_template(self) -> go.layout.Template:
        """
        Gets the plotly template associated with the theme.

        :return: The plotly template.
        :rtype: go.layout.Template
        """
        bg_color: str = self.armonia_theme.get_color("background").to_css_hsl(legacy=True)
        bg_inverted_color: str = self.armonia_theme.get_color("background_inverted").to_css_hsl(legacy=True)
        bg_inverted_mild_color: str = self.armonia_theme.get_color("background_inverted_mild").to_css_hsl(legacy=True)
        bg_inverted_milder_color: str = self.armonia_theme.get_color("background_inverted_milder").to_css_hsl(legacy=True)
        bg_inverted_faint_color: str = self.armonia_theme.get_color("background_inverted_faint").to_css_hsl(legacy=True)
        template: go.layout.Template = copy.deepcopy(pio.templates["plotly_white"])
        template.layout.paper_bgcolor = bg_color
        template.layout.plot_bgcolor = bg_color
        template.layout.font.color = bg_inverted_color
        template.layout.xaxis.gridcolor = bg_inverted_faint_color
        template.layout.xaxis.linecolor = bg_inverted_milder_color
        template.layout.xaxis.zerolinecolor = bg_inverted_mild_color
        template.layout.xaxis.tickfont.color = bg_inverted_mild_color
        template.layout.yaxis.gridcolor = bg_inverted_faint_color
        template.layout.yaxis.linecolor = bg_inverted_milder_color
        template.layout.yaxis.zerolinecolor = bg_inverted_mild_color
        template.layout.yaxis.tickfont.color = bg_inverted_mild_color
        template.layout.shapedefaults.line.color = bg_inverted_faint_color
        template.layout.geo.countrycolor = bg_inverted_faint_color
        template.layout.geo.lakecolor = bg_inverted_faint_color
        template.layout.geo.oceancolor = bg_inverted_faint_color
        template.layout.geo.landcolor = bg_color
        template.layout.geo.coastlinecolor = bg_inverted_color
        template.layout.geo.showcoastlines = True
        template.data["bar"][0].marker.line.color = bg_inverted_faint_color
        template.data["barpolar"][0].marker.line.color = bg_inverted_faint_color
        template.data["choropleth"][0].marker.line.color = bg_inverted_color
        template.data["choropleth"][0].marker.line.width = 0.5
        return template


__all__ = [
    "Theme",
]



"""
Chart options definitions.
"""

from abc import ABC, abstractmethod
from typing import Any

from orama.enums import GeoProjection, LineInterpolation, LinePlacement, LineSortOrder, SortOrder


class ChartOption(ABC):
    """
    Abstract base class for chart options.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        icon: str | None = None,
    ) -> None:
        self._name = name
        self._description = description
        self._map_to = map_to
        self._icon = icon

    @abstractmethod
    def validate(
        self,
        value: Any,
    ) -> None:
        """
        Validates a value for the option.

        :param value: The value to validate.
        :type value: Any
        """

    def describe(self) -> dict[str, Any]:
        """
        Provides a descriptive structure of the option.

        :return: A dictionary describing the option.
        :rtype: dict[str, Any]
        """
        return {
            "type": self.__class__,
            "name": self._name,
            "description": self._description,
            "map_to": self._map_to,
            "icon": self._icon,
        }


class BooleanOption(ChartOption):
    """
    Represents a boolean option for charts.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        default_value: bool = False,
        icon: str | None = None,
    ) -> None:
        """
        Constructor method.

        :param name: Name of the option.
        :type name: str
        :param description: Description of the option.
        :type description: str
        :param map_to: The internal parameter to map the variable to.
        :type map_to: str
        :param default_value: The default value for the option. Defaults to ``False``.
        :type default_value: bool, optional
        :param icon: An optional SVG icon string. Defaults to ``None``.
        :type icon: str | None, optional
        """
        super().__init__(
            name=name,
            description=description,
            map_to=map_to,
            icon=icon,
        )
        self._default_value = default_value

    def validate(
        self,
        value: Any,
    ) -> None:
        if not isinstance(value, bool):
            raise ValueError(f"Value {str(value)} for option {self._name} is not a boolean")

    def describe(self) -> dict[str, Any]:
        base_description = super().describe()
        base_description["default_value"] = self._default_value
        base_description["value_type"] = bool
        return base_description


class FloatOption(ChartOption):
    """
    Represents a float option for charts.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        default_value: float | None = 0.0,
        min_value: float | None = None,
        max_value: float | None = None,
        icon: str | None = None,
    ) -> None:
        """
        Constructor method.

        :param name: The name of the option.
        :type name: str
        :param description: The description of the option.
        :type description: str
        :param map_to: The internal parameter to map the variable to.
        :type map_to: str
        :param default_value: The default value for the option. Defaults to ``0.0``.
        :type default_value: float | None, optional
        :param min_value: The minimum value of the option, if any. Defaults to ``None``.
        :type min_value: float | None, optional
        :param max_value: The maximum value of the option, if any. Defaults to ``None``.
        :type max_value: float | None, optional
        :param icon: An optional SVG icon string. Defaults to ``None``.
        :type icon: str | None, optional
        """
        super().__init__(
            name=name,
            description=description,
            map_to=map_to,
            icon=icon,
        )
        self._default_value = default_value
        self._min_value = min_value
        self._max_value = max_value

    def validate(
        self,
        value: Any,
    ) -> None:
        if not isinstance(value, (float, int)):
            raise ValueError(f"Value {str(value)} for option {self._name} is not a float")
        if self._min_value is not None and value < self._min_value:
            raise ValueError(f"Value {str(value)} for option {self._name} is less than minimum {self._min_value}")
        if self._max_value is not None and value > self._max_value:
            raise ValueError(f"Value {str(value)} for option {self._name} is greater than maximum {self._max_value}")

    def describe(self) -> dict[str, Any]:
        base_description = super().describe()
        base_description["default_value"] = self._default_value
        base_description["value_type"] = float
        if self._min_value is not None:
            base_description["min_value"] = self._min_value
        if self._max_value is not None:
            base_description["max_value"] = self._max_value
        return base_description


class IntOption(ChartOption):
    """
    Represents an integer option for charts.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        default_value: int | None = 0,
        min_value: int | None = None,
        max_value: int | None = None,
        icon: str | None = None,
    ) -> None:
        """
        Constructor method.

        :param name: The name of the option.
        :type name: str
        :param description: The description of the option.
        :type description: str
        :param map_to: The internal parameter to map the variable to.
        :type map_to: str
        :param default_value: The default value for the option. Defaults to ``0``.
        :type default_value: int | None, optional
        :param min_value: The minimum value of the option, if any. Defaults to ``None``.
        :type min_value: int | None, optional
        :param max_value: The maximum value of the option, if any. Defaults to ``None``.
        :type max_value: int | None, optional
        :param icon: An optional SVG icon string. Defaults to ``None``.
        :type icon: str | None, optional
        """
        super().__init__(
            name=name,
            description=description,
            map_to=map_to,
            icon=icon,
        )
        self._default_value = default_value
        self._min_value = min_value
        self._max_value = max_value

    def validate(
        self,
        value: Any,
    ) -> None:
        if not isinstance(value, int) or isinstance(value, bool):
            raise ValueError(f"Value {str(value)} for option {self._name} is not an integer")
        if self._min_value is not None and value < self._min_value:
            raise ValueError(f"Value {str(value)} for option {self._name} is less than minimum {self._min_value}")
        if self._max_value is not None and value > self._max_value:
            raise ValueError(f"Value {str(value)} for option {self._name} is greater than maximum {self._max_value}")

    def describe(self) -> dict[str, Any]:
        base_description = super().describe()
        base_description["default_value"] = self._default_value
        base_description["value_type"] = int
        if self._min_value is not None:
            base_description["min_value"] = self._min_value
        if self._max_value is not None:
            base_description["max_value"] = self._max_value
        return base_description


class StringOption(ChartOption):
    """
    Represents a string option for charts.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        default_value: str | None = None,
        icon: str | None = None,
    ) -> None:
        """
        Constructor method.

        :param name: Name of the option.
        :type name: str
        :param description: Description of the option.
        :type description: str
        :param map_to: The internal parameter to map the variable to.
        :type map_to: str
        :param default_value: The default value for the option. Defaults to ``None``.
        :type default_value: str | None, optional
        :param icon: An optional SVG icon string. Defaults to ``None``.
        :type icon: str | None, optional
        """
        super().__init__(
            name=name,
            description=description,
            map_to=map_to,
            icon=icon,
        )
        self._default_value = default_value

    def validate(
        self,
        value: Any,
    ) -> None:
        if not isinstance(value, str):
            raise ValueError(f"Value {str(value)} for option {self._name} is not a string")

    def describe(self) -> dict[str, Any]:
        base_description = super().describe()
        base_description["default_value"] = self._default_value
        base_description["value_type"] = str
        return base_description


class SortOrderOption(ChartOption):
    """
    Represents a sort order option for charts.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        default_value: SortOrder = SortOrder.NONE,
        icon: str | None = None,
    ) -> None:
        """
        Constructor method.

        :param name: The name of the option.
        :type name: str
        :param description: The description of the option.
        :type description: str
        :param map_to: The internal parameter to map the variable to.
        :type map_to: str
        :param default_value: The default sorting order. Defaults to ``SortOrder.NONE``.
        :type default_value: SortOrder, optional
        :param icon: An optional SVG icon string. Defaults to ``None``.
        :type icon: str | None, optional
        """
        super().__init__(
            name=name,
            description=description,
            map_to=map_to,
            icon=icon,
        )
        self._default_value = default_value

    def validate(
        self,
        value: Any,
    ) -> None:
        if not isinstance(value, SortOrder):
            raise ValueError(f"Value {str(value)} for option {self._name} is not a sorting order")

    def describe(self) -> dict[str, Any]:
        base_description = super().describe()
        base_description["default_value"] = self._default_value
        base_description["value_type"] = SortOrder
        return base_description


class LinePlacementOption(ChartOption):
    """
    Represents a line placement mode option for charts.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        default_value: LinePlacement = LinePlacement.OVERLAPPING,
        icon: str | None = None,
    ) -> None:
        """
        Constructor method.

        :param name: The name of the option.
        :type name: str
        :param description: The description of the option.
        :type description: str
        :param map_to: The internal parameter to map the variable to.
        :type map_to: str
        :param default_value: The default placement mode. Defaults to ``LinePlacement.OVERLAPPING``.
        :type default_value: LinePlacement, optional
        :param icon: An optional SVG icon string. Defaults to ``None``.
        :type icon: str | None, optional
        """
        super().__init__(
            name=name,
            description=description,
            map_to=map_to,
            icon=icon,
        )
        self._default_value = default_value

    def validate(
        self,
        value: Any,
    ) -> None:
        if not isinstance(value, LinePlacement):
            raise ValueError(f"Value {str(value)} for option {self._name} is not a LinePlacement")

    def describe(self) -> dict[str, Any]:
        base_description = super().describe()
        base_description["default_value"] = self._default_value
        base_description["value_type"] = LinePlacement
        return base_description


class LineInterpolationOption(ChartOption):
    """
    Represents a line interpolation mode option for charts.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        default_value: LineInterpolation = LineInterpolation.LINEAR,
        icon: str | None = None,
    ) -> None:
        """
        Constructor method.

        :param name: The name of the option.
        :type name: str
        :param description: The description of the option.
        :type description: str
        :param map_to: The internal parameter to map the variable to.
        :type map_to: str
        :param default_value: The default interpolation mode. Defaults to ``LineInterpolation.LINEAR``.
        :type default_value: LineInterpolation, optional
        :param icon: An optional SVG icon string. Defaults to ``None``.
        :type icon: str | None, optional
        """
        super().__init__(
            name=name,
            description=description,
            map_to=map_to,
            icon=icon,
        )
        self._default_value = default_value

    def validate(
        self,
        value: Any,
    ) -> None:
        if not isinstance(value, LineInterpolation):
            raise ValueError(f"Value {str(value)} for option {self._name} is not a LineInterpolation")

    def describe(self) -> dict[str, Any]:
        base_description = super().describe()
        base_description["default_value"] = self._default_value
        base_description["value_type"] = LineInterpolation
        return base_description


class LineSortOrderOption(ChartOption):
    """
    Represents a sort order option for line chart series/categories.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        default_value: LineSortOrder = LineSortOrder.NONE,
        icon: str | None = None,
    ) -> None:
        """
        Constructor method.

        :param name: The name of the option.
        :type name: str
        :param description: The description of the option.
        :type description: str
        :param map_to: The internal parameter to map the variable to.
        :type map_to: str
        :param default_value: The default sort order. Defaults to ``LineSortOrder.NONE``.
        :type default_value: LineSortOrder, optional
        :param icon: An optional SVG icon string. Defaults to ``None``.
        :type icon: str | None, optional
        """
        super().__init__(
            name=name,
            description=description,
            map_to=map_to,
            icon=icon,
        )
        self._default_value = default_value

    def validate(
        self,
        value: Any,
    ) -> None:
        if not isinstance(value, LineSortOrder):
            raise ValueError(f"Value {value!r} for option {self._name} is not a LineSortOrder")

    def describe(self) -> dict[str, Any]:
        base_description = super().describe()
        base_description["default_value"] = self._default_value
        base_description["value_type"] = LineSortOrder
        return base_description


class GeoProjectionOption(ChartOption):
    """
    Represents a geo projection option for choropleth map charts.
    """

    def __init__(
        self,
        name: str,
        description: str,
        map_to: str,
        default_value: GeoProjection = GeoProjection.NATURAL_EARTH,
        icon: str | None = None,
    ) -> None:
        """
        Constructor method.

        :param name: The name of the option.
        :type name: str
        :param description: The description of the option.
        :type description: str
        :param map_to: The internal parameter to map the variable to.
        :type map_to: str
        :param default_value: The default geo projection. Defaults to ``GeoProjection.NATURAL_EARTH``.
        :type default_value: GeoProjection, optional
        :param icon: An optional SVG icon string. Defaults to ``None``.
        :type icon: str | None, optional
        """
        super().__init__(
            name=name,
            description=description,
            map_to=map_to,
            icon=icon,
        )
        self._default_value = default_value

    def validate(
        self,
        value: Any,
    ) -> None:
        if not isinstance(value, GeoProjection):
            raise ValueError(f"Value {str(value)} for option {self._name} is not a GeoProjection")

    def describe(self) -> dict[str, Any]:
        base_description = super().describe()
        base_description["default_value"] = self._default_value
        base_description["value_type"] = GeoProjection
        return base_description


ChartOptionsMap = dict[str, ChartOption]


__all__ = [
    "ChartOption",
    "BooleanOption",
    "FloatOption",
    "IntOption",
    "StringOption",
    "SortOrderOption",
    "LinePlacementOption",
    "LineInterpolationOption",
    "LineSortOrderOption",
    "GeoProjectionOption",
    "ChartOptionsMap",
]
